@@ -180,7 +180,8 @@ static const GDebugKey clutter_paint_debug_keys[] = {
   { "disable-clipped-redraws", CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS },
   { "redraws", CLUTTER_DEBUG_REDRAWS },
   { "paint-volumes", CLUTTER_DEBUG_PAINT_VOLUMES },
-  { "disable-culling", CLUTTER_DEBUG_DISABLE_CULLING }
+  { "disable-culling", CLUTTER_DEBUG_DISABLE_CULLING },
+  { "disable-offscreen-redirect", CLUTTER_DEBUG_DISABLE_OFFSCREEN_REDIRECT }
 };
 
 #ifdef CLUTTER_ENABLE_PROFILE
@@ -246,8 +247,8 @@ clutter_redraw (ClutterStage *stage)
  * clutter_set_motion_events_enabled:
  * @enable: %TRUE to enable per-actor motion events
  *
- * Sets whether per-actor motion events should be enabled or not (the
- * default is to enable them).
+ * Sets whether per-actor motion events should be enabled or not on
+ * all #ClutterStage<!-- -->s managed by Clutter.
  *
  * If @enable is %FALSE the following events will not work:
  * <itemizedlist>
@@ -258,6 +259,8 @@ clutter_redraw (ClutterStage *stage)
  * </itemizedlist>
  *
  * Since: 0.6
+ *
+ * Deprecated: 1.8: Use clutter_stage_set_motion_events_enabled() instead.
  */
 void
 clutter_set_motion_events_enabled (gboolean enable)
@@ -282,7 +285,7 @@ clutter_set_motion_events_enabled (gboolean enable)
        l != NULL;
        l = l->next)
     {
-      _clutter_stage_set_motion_events_enabled (l->data, enable);
+      clutter_stage_set_motion_events_enabled (l->data, enable);
     }
 }
 
@@ -294,13 +297,13 @@ clutter_set_motion_events_enabled (gboolean enable)
  * Return value: %TRUE if the motion events are enabled
  *
  * Since: 0.6
+ *
+ * Deprecated: 1.8: Use clutter_stage_get_motion_events_enabled() instead.
  */
 gboolean
 clutter_get_motion_events_enabled (void)
 {
-  ClutterMainContext *context = _clutter_context_get_default ();
-
-  return context->motion_events_per_actor;
+  return _clutter_context_get_motion_events_enabled ();
 }
 
 ClutterActor *
@@ -1235,7 +1238,10 @@ clutter_init_real (GError **error)
 
   /* The same is true when drawing the outlines of paint volumes... */
   if (clutter_paint_debug_flags & CLUTTER_DEBUG_PAINT_VOLUMES)
-    clutter_paint_debug_flags |= CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS;
+    {
+      clutter_paint_debug_flags |=
+        CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS | CLUTTER_DEBUG_DISABLE_CULLING;
+    }
 
   /* this will take care of initializing Cogl's state and
    * query the GL machinery for features
@@ -2128,7 +2134,7 @@ _clutter_process_event_details (ClutterActor        *stage,
 
       case CLUTTER_MOTION:
         /* only the stage gets motion events if they are enabled */
-        if (!_clutter_stage_get_motion_events_enabled (CLUTTER_STAGE (stage)) &&
+        if (!clutter_stage_get_motion_events_enabled (CLUTTER_STAGE (stage)) &&
             event->any.source == NULL)
           {
             /* Only stage gets motion events */
@@ -3028,7 +3034,7 @@ _clutter_context_release_id (guint32 id_)
 {
   ClutterMainContext *context = _clutter_context_get_default ();
 
-  return _clutter_id_pool_remove (context->id_pool, id_);
+  _clutter_id_pool_remove (context->id_pool, id_);
 }
 
 void
@@ -3091,3 +3097,11 @@ _clutter_context_pop_shader_stack (ClutterActor *actor)
 
   return _clutter_context_peek_shader_stack ();
 }
+
+gboolean
+_clutter_context_get_motion_events_enabled (void)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+
+  return context->motion_events_per_actor;
+}
