@@ -2,6 +2,7 @@
 #include <clutter/clutter.h>
 #include <cogl/cogl.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include "test-conform-common.h"
 
@@ -14,10 +15,10 @@ static const ClutterColor stage_color = { 0x0, 0x0, 0x0, 0xff };
 #define BLUE 2
 #define ALPHA 3
 
-#define MASK_RED(COLOR)   ((COLOR & 0xff000000) >> 24);
-#define MASK_GREEN(COLOR) ((COLOR & 0xff0000) >> 16);
-#define MASK_BLUE(COLOR)  ((COLOR & 0xff00) >> 8);
-#define MASK_ALPHA(COLOR) (COLOR & 0xff);
+#define MASK_RED(COLOR)   ((COLOR & 0xff000000) >> 24)
+#define MASK_GREEN(COLOR) ((COLOR & 0xff0000) >> 16)
+#define MASK_BLUE(COLOR)  ((COLOR & 0xff00) >> 8)
+#define MASK_ALPHA(COLOR) (COLOR & 0xff)
 
 #define BLEND_CONSTANT_UNUSED 0xDEADBEEF
 #define TEX_CONSTANT_UNUSED   0xDEADBEEF
@@ -39,10 +40,9 @@ check_pixel (GLubyte *pixel, guint32 color)
   if (g_test_verbose ())
     g_print ("  expected = %x, %x, %x, %x\n",
              r, g, b, a);
-  /* FIXME - allow for hardware in-precision */
-  g_assert_cmpint (pixel[RED], ==, r);
-  g_assert_cmpint (pixel[GREEN], ==, g);
-  g_assert_cmpint (pixel[BLUE], ==, b);
+  g_assert_cmpint (abs(pixel[RED] - r), <=, 1);
+  g_assert_cmpint (abs(pixel[GREEN] - g), <=, 1);
+  g_assert_cmpint (abs(pixel[BLUE] - b), <=, 1);
 
   /* FIXME
    * We ignore the alpha, since we don't know if our render target is
