@@ -54,7 +54,8 @@ test_texture_quality_main (int argc, char *argv[])
   GError           *error;
   gchar            *file;
 
-  clutter_init (&argc, &argv);
+  if (clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS)
+    return 1;
 
   stage = clutter_stage_get_default ();
   clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);
